c gen_fakehurr - Plot3D and VTK mesh and solution generator
c                (fake hurricane)
c
c
c Copyright (c) 2006 Francesco Poli <frx@firenze.linux.it>
c 
c Permission is hereby granted, free of charge, to any person obtaining
c a copy of this software and associated documentation files (the
c "Software"), to deal in the Software without restriction, including
c without limitation the rights to use, copy, modify, merge, publish,
c distribute, sublicense, and/or sell copies of the Software, and to
c permit persons to whom the Software is furnished to do so, subject to
c the following conditions:
c 
c The above copyright notice and this permission notice shall be
c included in all copies or substantial portions of the Software.
c 
c THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
c EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
c MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
c IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
c CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
c TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
c SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


      program main


c     constant definitions:

      integer imm,jmm,kmm,ngrid
      parameter (imm=46,jmm=31,kmm=41,ngrid=2)


c     variable declarations:

      character*32 filenm
      real rr(jmm),th(imm,ngrid),zz(kmm)
      real x(imm,jmm,kmm,ngrid),y(imm,jmm,kmm,ngrid)
      real z(imm,jmm,kmm,ngrid),p(imm,jmm,kmm,ngrid)
      real q(imm,jmm,kmm,5,ngrid)
      real qmer(jmm,kmm,4,ngrid),qcro(imm,jmm,4,ngrid)
      integer im(ngrid),jm(ngrid),km(ngrid)
      real pi,rmax,rmin,rfact,delth,height,delzz,xl,yl,zl
      integer i,j,igrid,k,nx,kcro
      real ga,rgas,machm,ptot,ttot,rotot,drdz,dtdl,tauzz,rapex
      real mach,ar,ttont,temp,ro,pres,wsq,wlsq,wt,wr,wz,wy,wx,ut
      real utmer,utcro
      real fsmach,alpha,re,time


c     ================================================================
c     This little program generates a 3D mesh and a 3D flow field that
c     describes something that  more or less resembles to a hurricane;
c     then writes data in both PLOT3D and VTK formats.  It also writes
c     some 2D sections in both formats.
c
c     WARNING!
c     The generated data are *not* an accurate physical model, nor aim
c     to be useful for any atmospheric physics prediction.
c     ================================================================


c     pi

      pi = 4.0*atan(1.0)


c     generate mesh and solution:

      im(1) = imm
      jm(1) = jmm
      km(1) = kmm
      im(2) = imm
      jm(2) = jmm
      km(2) = kmm

      kcro = (kmm - 1)/2 + 1

      rmax  = 6.0
      rmin  = 0.1
      rfact = (rmax/rmin)**(1.0/(jmm - 1.0))
      do 5 j = 1, jmm
        rr(j) = rmin*(rfact**(j-1))
 5    continue

      delth = pi/(imm - 1.0)
      do 6 i = 1, imm
        th(i,1) = (i - 1.0)*delth
        th(i,2) = pi + th(i,1)
 6    continue

      height = 9.0
      delzz  = height/(kmm - 1.0)
      do 7 k = 1, kmm
        zz(k) = (k - 1.0)*delzz
 7    continue

      ga    = 1.4
      rgas  = 1.0
      machm = 0.2
      ptot  = 1.0
      ttot  = 1.0
      rotot = ptot/(rgas*ttot)

      drdz  = 0.04
      dtdl  = 10.0
      tauzz = 0.1
      rapex = 1.0

      do 100 igrid = 1, 2

        do 10 k = 1, km(igrid)
        do 10 j = 1, jm(igrid)
        do 10 i = 1, im(igrid)

          xl = rr(j)*cos(th(i,igrid))
          yl = rr(j)*sin(th(i,igrid))
          zl = zz(k)

          mach = machm
          mach = (2.0/pi)*atan(zz(k)/tauzz)*mach
          ar   = rr(j)/(rapex + drdz*zz(k))
          mach = 20.0*(ar**3)*exp(-3*ar)*mach

          ttont = 1.0 + 0.5*(ga - 1.0)*(mach**2)
          temp  =  ttot/ ttont
          ro    = rotot/(ttont**(1.0/(ga - 1.0)))
          pres  =  ptot/(ttont**( ga/(ga - 1.0)))

          wsq  = ga*rgas*temp*(mach**2)
          wlsq = wsq/(dtdl**2 + 1.0)
          wt   = dtdl*sqrt(wlsq)
          wz   = sqrt(wlsq/(drdz**2 + 1.0))
          wr   = drdz*wz
          wx   = wr*cos(th(i,igrid)) - wt*sin(th(i,igrid))
          wy   = wr*sin(th(i,igrid)) + wt*cos(th(i,igrid))

          ut = rgas*temp/(ga - 1.0) + 0.5*wsq

          x(i,j,k,  igrid) = xl
          y(i,j,k,  igrid) = yl
          z(i,j,k,  igrid) = zl
          p(i,j,k,  igrid) = pres
          q(i,j,k,1,igrid) = ro
          q(i,j,k,2,igrid) = ro*wx
          q(i,j,k,3,igrid) = ro*wy
          q(i,j,k,4,igrid) = ro*wz
          q(i,j,k,5,igrid) = ro*ut

          if (i.eq.1) then
            utmer = rgas*temp/(ga - 1.0) + 0.5*wlsq
            qmer(j,k,1,igrid) = ro
            qmer(j,k,2,igrid) = ro*wx
            qmer(j,k,3,igrid) = ro*wz
            qmer(j,k,4,igrid) = ro*utmer
          end if

          if (k.eq.kcro) then
            utcro = rgas*temp/(ga - 1.0) + 0.5*(wx**2 + wy**2)
            qcro(i,j,1,igrid) = ro
            qcro(i,j,2,igrid) = ro*wx
            qcro(i,j,3,igrid) = ro*wy
            qcro(i,j,4,igrid) = ro*utcro
          end if

 10     continue

 100  continue

      
c     write out 3D mesh Plot3D file:

      filenm = 'fakehurr_3D.xyz'
      open (unit=1,file=filenm,form='unformatted')
      rewind 1
      write(1) ngrid
      write(1) (im(igrid),jm(igrid),km(igrid), igrid=1,ngrid)
      do 80 igrid = 1, ngrid
        write(1)
     $  (((x(i,j,k,igrid),
     $     i=1,im(igrid)), j=1,jm(igrid)), k=1,km(igrid)),
     $  (((y(i,j,k,igrid),
     $     i=1,im(igrid)), j=1,jm(igrid)), k=1,km(igrid)),
     $  (((z(i,j,k,igrid),
     $     i=1,im(igrid)), j=1,jm(igrid)), k=1,km(igrid))
 80   continue
      close(unit=1)


c     write out 3D solution Plot3D file:

      filenm = 'fakehurr_3D.q'
      fsmach = machm
      alpha  = 0.0
      re     = 1.0e5
      time   = 0.0
      open (unit=1,file=filenm,form='unformatted')
      rewind 1
      write(1) ngrid
      write(1) (im(igrid),jm(igrid),km(igrid), igrid=1,ngrid)
      do 90 igrid = 1, ngrid
        write(1) fsmach,alpha,re,time
        write(1)
     $  ((((q(i,j,k,nx,igrid),
     $      i=1,im(igrid)), j=1,jm(igrid)), k=1,km(igrid)), nx=1,5)
 90   continue
      close(unit=1)


c     write out 3D VTK data file:

      filenm = 'fakehurr_3D.vtk'
      open (unit=1,file=filenm,form='formatted')
      rewind 1
      write(1,'(a)') '# vtk DataFile Version 2.0'
      write(1,'(a)') 'fake hurricane (generated by gen_fakehurr)'
      write(1,'(a)') 'ASCII'

      write(1,*)
      write(1,'(a)') 'DATASET STRUCTURED_GRID'
      write(1,'(a, 3 i6)') 'DIMENSIONS ', 2*imm - 1, jmm, kmm
      write(1,'(a, i12, a)') 'POINTS ', (2*imm - 1)*jmm*kmm, ' float'
      do 1000 k = 1, kmm
      do 1000 j = 1, jmm
        do 1010 igrid = 1, 2
        do 1010 i = 1, imm-1
          write(1,*) x(i,j,k,igrid), y(i,j,k,igrid), z(i,j,k,igrid)
 1010   continue
        igrid = 2
        i = imm
        write(1,*) x(i,j,k,igrid), y(i,j,k,igrid), z(i,j,k,igrid)
 1000 continue

      write(1,*)
      write(1,'(a, i12)') 'POINT_DATA ', (2*imm - 1)*jmm*kmm
      write(1,'(a)') 'SCALARS pressure float 1'
      write(1,'(a)') 'LOOKUP_TABLE default'
      do 1100 k = 1, kmm
      do 1100 j = 1, jmm
        do 1110 igrid = 1, 2
        do 1110 i = 1, imm-1
          write(1,*) p(i,j,k,igrid)
 1110   continue
        igrid = 2
        i = imm
        write(1,*) p(i,j,k,igrid)
 1100 continue
      write(1,'(a)') 'SCALARS density float 1'
      write(1,'(a)') 'LOOKUP_TABLE default'
      do 1200 k = 1, kmm
      do 1200 j = 1, jmm
        do 1210 igrid = 1, 2
        do 1210 i = 1, imm-1
          write(1,*) q(i,j,k,1,igrid)
 1210   continue
        igrid = 2
        i = imm
        write(1,*) q(i,j,k,1,igrid)
 1200 continue
      write(1,'(a)') 'VECTORS velocity float'
      do 1300 k = 1, kmm
      do 1300 j = 1, jmm
        do 1310 igrid = 1, 2
        do 1310 i = 1, imm-1
          write(1,*) q(i,j,k,2,igrid)/q(i,j,k,1,igrid),
     $               q(i,j,k,3,igrid)/q(i,j,k,1,igrid),
     $               q(i,j,k,4,igrid)/q(i,j,k,1,igrid)
 1310   continue
        igrid = 2
        i = imm
        write(1,*) q(i,j,k,2,igrid)/q(i,j,k,1,igrid),
     $             q(i,j,k,3,igrid)/q(i,j,k,1,igrid),
     $             q(i,j,k,4,igrid)/q(i,j,k,1,igrid)
 1300 continue

      close(unit=1)

      
c     write out 2D meridional mesh Plot3D file:

      filenm = 'fakehurr_merid.xyz'
      igrid = 1
      i = 1
      open (unit=1,file=filenm,form='unformatted')
      rewind 1
      write(1) 1
      write(1) jm(igrid),km(igrid)
      write(1)
     $((x(i,j,k,igrid),
     $  j=1,jm(igrid)), k=1,km(igrid)),
     $((z(i,j,k,igrid),
     $  j=1,jm(igrid)), k=1,km(igrid))
      close(unit=1)


c     write out 2D meridional solution Plot3D file:

      filenm = 'fakehurr_merid.q'
      igrid = 1
      i = 1
      fsmach = machm
      alpha  = 0.0
      re     = 1.0e5
      time   = 0.0
      open (unit=1,file=filenm,form='unformatted')
      rewind 1
      write(1) 1
      write(1) jm(igrid),km(igrid)
      write(1) fsmach,alpha,re,time
      write(1)
     $(((qmer(j,k,nx,igrid),
     $   j=1,jm(igrid)), k=1,km(igrid)), nx=1,4)
      close(unit=1)


c     write out 2D meridional VTK data file:

      filenm = 'fakehurr_merid.vtk'
      igrid = 1
      i = 1
      open (unit=1,file=filenm,form='formatted')
      rewind 1
      write(1,'(a)') '# vtk DataFile Version 2.0'
      write(1,'(a)') 'fake hurricane (generated by gen_fakehurr)'
      write(1,'(a)') 'ASCII'

      write(1,*)
      write(1,'(a)') 'DATASET STRUCTURED_GRID'
      write(1,'(a, 3 i6)') 'DIMENSIONS ', jmm, kmm, 1
      write(1,'(a, i12, a)') 'POINTS ', jmm*kmm*1, ' float'
      do 2000 k = 1, kmm
      do 2000 j = 1, jmm
        write(1,*) x(i,j,k,igrid), z(i,j,k,igrid), 0.0
 2000 continue

      write(1,*)
      write(1,'(a, i12)') 'POINT_DATA ', jmm*kmm*1
      write(1,'(a)') 'SCALARS pressure float 1'
      write(1,'(a)') 'LOOKUP_TABLE default'
      do 2100 k = 1, kmm
      do 2100 j = 1, jmm
        write(1,*) p(i,j,k,igrid)
 2100 continue
      write(1,'(a)') 'SCALARS density float 1'
      write(1,'(a)') 'LOOKUP_TABLE default'
      do 2200 k = 1, kmm
      do 2200 j = 1, jmm
        write(1,*) qmer(j,k,1,igrid)
 2200 continue
      write(1,'(a)') 'VECTORS velocity float'
      do 2300 k = 1, kmm
      do 2300 j = 1, jmm
        write(1,*) qmer(j,k,2,igrid)/qmer(j,k,1,igrid),
     $             qmer(j,k,3,igrid)/qmer(j,k,1,igrid),
     $             0.0
 2300 continue

      close(unit=1)

      
c     write out 2D cross-section mesh Plot3D file:

      filenm = 'fakehurr_cross.xyz'
      k = kcro
      open (unit=1,file=filenm,form='unformatted')
      rewind 1
      write(1) ngrid
      write(1) (im(igrid),jm(igrid), igrid=1,ngrid)
      do 3080 igrid = 1, ngrid
        write(1)
     $  ((x(i,j,k,igrid),
     $    i=1,im(igrid)), j=1,jm(igrid)),
     $  ((y(i,j,k,igrid),
     $    i=1,im(igrid)), j=1,jm(igrid))
 3080 continue
      close(unit=1)


c     write out 2D cross-section solution Plot3D file:

      filenm = 'fakehurr_cross.q'
      k = kcro
      fsmach = machm
      alpha  = 0.0
      re     = 1.0e5
      time   = 0.0
      open (unit=1,file=filenm,form='unformatted')
      rewind 1
      write(1) ngrid
      write(1) (im(igrid),jm(igrid), igrid=1,ngrid)
      do 3090 igrid = 1, ngrid
        write(1) fsmach,alpha,re,time
        write(1)
     $  (((qcro(i,j,nx,igrid),
     $     i=1,im(igrid)), j=1,jm(igrid)), nx=1,4)
 3090 continue
      close(unit=1)


c     write out 2D cross-section VTK data file:

      filenm = 'fakehurr_cross.vtk'
      k = kcro
      open (unit=1,file=filenm,form='formatted')
      rewind 1
      write(1,'(a)') '# vtk DataFile Version 2.0'
      write(1,'(a)') 'fake hurricane (generated by gen_fakehurr)'
      write(1,'(a)') 'ASCII'

      write(1,*)
      write(1,'(a)') 'DATASET STRUCTURED_GRID'
      write(1,'(a, 3 i6)') 'DIMENSIONS ', 2*imm - 1, jmm, 1
      write(1,'(a, i12, a)') 'POINTS ', (2*imm - 1)*jmm*1, ' float'
      do 3000 j = 1, jmm
        do 3010 igrid = 1, 2
        do 3010 i = 1, imm-1
          write(1,*) x(i,j,k,igrid), y(i,j,k,igrid), 0.0
 3010   continue
        igrid = 2
        i = imm
        write(1,*) x(i,j,k,igrid), y(i,j,k,igrid), 0.0
 3000 continue

      write(1,*)
      write(1,'(a, i12)') 'POINT_DATA ', (2*imm - 1)*jmm*1
      write(1,'(a)') 'SCALARS pressure float 1'
      write(1,'(a)') 'LOOKUP_TABLE default'
      do 3100 j = 1, jmm
        do 3110 igrid = 1, 2
        do 3110 i = 1, imm-1
          write(1,*) p(i,j,k,igrid)
 3110   continue
        igrid = 2
        i = imm
        write(1,*) p(i,j,k,igrid)
 3100 continue
      write(1,'(a)') 'SCALARS density float 1'
      write(1,'(a)') 'LOOKUP_TABLE default'
      do 3200 j = 1, jmm
        do 3210 igrid = 1, 2
        do 3210 i = 1, imm-1
          write(1,*) qcro(i,j,1,igrid)
 3210   continue
        igrid = 2
        i = imm
        write(1,*) qcro(i,j,1,igrid)
 3200 continue
      write(1,'(a)') 'VECTORS velocity float'
      do 3300 j = 1, jmm
        do 3310 igrid = 1, 2
        do 3310 i = 1, imm-1
          write(1,*) qcro(i,j,2,igrid)/qcro(i,j,1,igrid),
     $               qcro(i,j,3,igrid)/qcro(i,j,1,igrid),
     $               0.0
 3310   continue
        igrid = 2
        i = imm
        write(1,*) qcro(i,j,2,igrid)/qcro(i,j,1,igrid),
     $             qcro(i,j,3,igrid)/qcro(i,j,1,igrid),
     $             0.0
 3300 continue

      close(unit=1)


      stop
      end
